@@ -3,13 +3,12 @@
  *
  * This program is licensed under BSD license, read COPYING
  *
- *  $Id: conky.h 757 2006-11-12 06:38:21Z mirrorbox $
+ *  $Id: conky.h 814 2006-12-12 23:21:14Z pkovacs $
  */
 
 #ifndef _conky_h_
 #define _conky_h_
 
-#include <pthread.h>
 #if defined(HAS_MCHECK_H)
 #include <mcheck.h>
 #endif /* HAS_MCHECK_H */
@@ -41,6 +40,8 @@
 #include <xmmsclient/xmmsclient.h>
 #endif
 
+#include "timed_thread.h"
+
 #define TOP_CPU 1
 #define TOP_NAME 2
 #define TOP_PID 3
@@ -49,6 +50,9 @@
 #define TEXT_BUFFER_SIZE 1280
 #define P_MAX_SIZE ((TEXT_BUFFER_SIZE * 4) - 2) 
 
+/* maximum number of special things, e.g. fonts, offsets, aligns, etc. */
+#define MAX_SPECIALS_DEFAULT 512
+
 /* maximum size of config TEXT buffer, i.e. below TEXT line. */
 #define MAX_USER_TEXT_DEFAULT 16384
 
@@ -96,11 +100,6 @@ struct fs_stat {
 	long long free;
 };
 
-struct thread_info_s {
-	pthread_t thread;
-	pthread_mutex_t mutex;
-};
-
 struct mail_s {			// for imap and pop3
 	unsigned long unseen;
 	unsigned long messages;
@@ -114,9 +113,8 @@ struct mail_s {			// for imap and pop3
 	char pass[128];
 	char command[1024];
 	char folder[128];
-	int pos;
-	struct thread_info_s thread_info;
 	char secure;
+	timed_thread *p_timed_thread;
 } mail;
 
 /*struct cpu_stat {
@@ -172,11 +170,8 @@ struct xmms2_s {
 #ifdef AUDACIOUS
 struct audacious_s {
 	audacious_t items;              /* e.g. items[AUDACIOUS_STATUS] */
-	int runnable;                   /* used to signal worker thread to stop */
-	pthread_t thread;               /* worker thread */
-	pthread_attr_t thread_attr;     /* thread attributes */
-	pthread_mutex_t item_mutex;     /* mutex for item array */
-	pthread_mutex_t runnable_mutex; /* mutex for runnable flag */
+	int max_title_len;		/* e.g. ${audacious_title 50} */
+	timed_thread *p_timed_thread;
 };
 #endif
 
@@ -192,10 +187,15 @@ struct bmpx_s {
 };
 #endif
 
+void update_entropy();
+struct entropy_s {
+	unsigned int entropy_avail;
+	unsigned int poolsize;
+};
+
 #ifdef TCP_PORT_MONITOR
 #include "libtcp-portmon.h"
-#define MIN_PORT_MONITORS_DEFAULT 16
-#define MIN_PORT_MONITOR_CONNECTIONS_DEFAULT 256
+#define MAX_PORT_MONITOR_CONNECTIONS_DEFAULT 256
 #endif
 
 enum {
@@ -232,9 +232,16 @@ enum {
 #ifdef XMMS2
 	INFO_XMMS2 = 22,
 #endif
+	INFO_ENTROPY = 23,
 };
 
 
+/* get_battery_stuff() item selector */
+enum {
+	BATTERY_STATUS,
+	BATTERY_TIME
+};
+
 #ifdef MPD
 #include "libmpdclient.h"
 #endif
@@ -251,8 +258,8 @@ struct information {
 	double uptime;
 
 	/* memory information in kilobytes */
-	unsigned long mem, memmax, swap, swapmax;
-	unsigned long bufmem, buffers, cached;
+	unsigned long long mem, memmax, swap, swapmax;
+	unsigned long long bufmem, buffers, cached;
 
 	unsigned short procs;
 	unsigned short run_procs;
@@ -294,6 +301,7 @@ struct information {
         tcp_port_monitor_collection_t * p_tcp_port_monitor_collection;
 #endif
 	short kflags;  /* kernel settings, see enum KFLAG */
+	struct entropy_s entropy;
 };
 
 enum {
@@ -458,7 +466,7 @@ int open_acpi_temperature(const char *name);
 double get_acpi_temperature(int fd);
 void get_acpi_ac_adapter( char *, size_t ); 
 void get_acpi_fan( char *, size_t ); 
-void get_battery_stuff(char *buf, unsigned int n, const char *bat);
+void get_battery_stuff(char *buf, unsigned int n, const char *bat, int item);
 void get_ibm_acpi_fan(char *buf, size_t client_buffer_size);
 void get_ibm_acpi_temps(void);
 void get_ibm_acpi_volume(char *buf, size_t client_buffer_size);
