@@ -1,7 +1,7 @@
 /* linux.c
  * Contains linux specific code
  *
- *  $Id: linux.c 738 2006-11-08 03:06:42Z pkovacs $
+ *  $Id: linux.c 814 2006-12-12 23:21:14Z pkovacs $
  */
 
 
@@ -113,17 +113,17 @@ void update_meminfo()
 			break;
 
 		if (strncmp(buf, "MemTotal:", 9) == 0) {
-			sscanf(buf, "%*s %lu", &info.memmax);
+			sscanf(buf, "%*s %Lu", &info.memmax);
 		} else if (strncmp(buf, "MemFree:", 8) == 0) {
-			sscanf(buf, "%*s %lu", &info.mem);
+			sscanf(buf, "%*s %Lu", &info.mem);
 		} else if (strncmp(buf, "SwapTotal:", 10) == 0) {
-			sscanf(buf, "%*s %lu", &info.swapmax);
+			sscanf(buf, "%*s %Lu", &info.swapmax);
 		} else if (strncmp(buf, "SwapFree:", 9) == 0) {
-			sscanf(buf, "%*s %lu", &info.swap);
+			sscanf(buf, "%*s %Lu", &info.swap);
 		} else if (strncmp(buf, "Buffers:", 8) == 0) {
-			sscanf(buf, "%*s %lu", &info.buffers);
+			sscanf(buf, "%*s %Lu", &info.buffers);
 		} else if (strncmp(buf, "Cached:", 7) == 0) {
-			sscanf(buf, "%*s %lu", &info.cached);
+			sscanf(buf, "%*s %Lu", &info.cached);
 		}
 	}
 	
@@ -1192,23 +1192,26 @@ static FILE *apm_bat_fp;
 
 static int acpi_last_full;
 
-static char last_battery_str[64];
+static char last_battery_str[64];	/* e.g. "charging 75%" */
+static char last_battery_time_str[64];	/* e.g. "3h 15m" */
 
 static double last_battery_time;
 
-void get_battery_stuff(char *buf, unsigned int n, const char *bat)
+void get_battery_stuff(char *buf, unsigned int n, const char *bat, int item)
 {
 	static int rep, rep2;
 	char acpi_path[128];
 	snprintf(acpi_path, 127, ACPI_BATTERY_BASE_PATH "/%s/state", bat);
 
 	/* don't update battery too often */
-	if (current_update_time - last_battery_time < 29.5) {
-		snprintf(buf, n, "%s", last_battery_str);
-		return;
-	}
+	if (current_update_time - last_battery_time < 29.5) 
+		goto set_return_value;	
+
 	last_battery_time = current_update_time;
 
+	memset (last_battery_str, 0, sizeof (last_battery_str));
+	memset (last_battery_time_str, 0, sizeof (last_battery_time_str));
+
 	/* first try ACPI */
 
 	if (acpi_bat_fp == NULL && apm_bat_fp == NULL)
@@ -1252,19 +1255,20 @@ void get_battery_stuff(char *buf, unsigned int n, const char *bat)
 				break;
 
 			/* let's just hope units are ok */
-			if (buf[0] == 'p') {
+			if (strncmp (buf, "present:", 8) == 0) 
 				sscanf(buf, "present: %4s", present);
-			} else if (buf[0] == 'c')
-				sscanf(buf, "charging state: %63s",
-				       charging_state);
-			else if (buf[0] == 'p')
-				sscanf(buf, "present rate: %d",
-				       &present_rate);
-			else if (buf[0] == 'r')
-				sscanf(buf, "remaining capacity: %d",
-				       &remaining_capacity);
+			else if (strncmp (buf, "charging state:", 15) == 0)
+				sscanf(buf, "charging state: %63s", charging_state);
+			else if (strncmp (buf, "present rate:", 13) == 0)
+				sscanf(buf, "present rate: %d", &present_rate);
+			else if (strncmp(buf, "remaining capacity:", 19) == 0)
+				sscanf(buf, "remaining capacity: %d", &remaining_capacity);
 		}
-		
+
+		/* Hellf[i]re notes that remaining capacity can exceed acpi_last_full */
+		if (remaining_capacity > acpi_last_full)
+			acpi_last_full = remaining_capacity;  /* normalize to 100% */
+
 		/* not present */
 		if (strcmp(present, "no") == 0) {
 			strncpy(last_battery_str, "not present", 64);
@@ -1272,35 +1276,35 @@ void get_battery_stuff(char *buf, unsigned int n, const char *bat)
 		/* charging */
 		else if (strcmp(charging_state, "charging") == 0) {
 			if (acpi_last_full != 0 && present_rate > 0) {
-				snprintf(last_battery_str, 63, "charging %i%% ", (int) (remaining_capacity / acpi_last_full) * 100);
-				format_seconds(last_battery_str + 14,
-					       63 - 14,
-					       (acpi_last_full -
-						remaining_capacity) * 60 *
-					       60 / present_rate);
-			} else if (acpi_last_full != 0
-				   && present_rate <= 0) {
-				snprintf(last_battery_str, 64, "charging %d%%",
-					remaining_capacity * 100 /
-					acpi_last_full);
+				/* e.g. charging 75% */
+				snprintf(last_battery_str, sizeof(last_battery_str)-1, "charging %i%%", 
+					(int) ((remaining_capacity * 100) / acpi_last_full));
+				/* e.g. 2h 37m */
+				format_seconds(last_battery_time_str, sizeof(last_battery_time_str)-1,
+					      (long) (((acpi_last_full - remaining_capacity) * 3600) / 
+					              present_rate));
+			} else if (acpi_last_full != 0 && present_rate <= 0) {
+				snprintf(last_battery_str, sizeof(last_battery_str)-1, "charging %d%%",
+					(int) ((remaining_capacity * 100) / acpi_last_full));
 			} else {
-				strncpy(last_battery_str, "charging", 63);
+				strncpy(last_battery_str, "charging", sizeof(last_battery_str)-1);
 			}
 		}
 		/* discharging */
 		else if (strncmp(charging_state, "discharging", 64) == 0) {
 			if (present_rate > 0) {
-				snprintf(last_battery_str, 63, "discharging %i%% ", (int)(remaining_capacity / acpi_last_full) * 100);
-				format_seconds(last_battery_str + 17, 63 - 17,
-					       (remaining_capacity * 60 *
-						60) / present_rate);
+				/* e.g. discharging 35% */
+				snprintf(last_battery_str, sizeof(last_battery_str)-1, "discharging %i%%",
+					(int) ((remaining_capacity * 100) / acpi_last_full));
+				/* e.g. 1h 12m */
+				format_seconds(last_battery_time_str, sizeof(last_battery_time_str)-1,
+					      (long) ((remaining_capacity * 3600) / present_rate));
 			} else if (present_rate == 0) { /* Thanks to Nexox for this one */
-				snprintf(last_battery_str, 64, "full");
+				snprintf(last_battery_str, sizeof(last_battery_str)-1, "full");
 			} else {
-				snprintf(last_battery_str, 64,
+				snprintf(last_battery_str, sizeof(last_battery_str)-1,
 					"discharging %d%%",
-					remaining_capacity * 100 /
-					acpi_last_full);
+					(int) ((remaining_capacity * 100) / acpi_last_full));
 			}
 		}
 		/* charged */
@@ -1313,8 +1317,7 @@ void get_battery_stuff(char *buf, unsigned int n, const char *bat)
 			if (acpi_last_full != 0
 			    && remaining_capacity != acpi_last_full)
 				snprintf(last_battery_str, 64, "unknown %d%%",
-					remaining_capacity * 100 /
-					acpi_last_full);
+					(int) ((remaining_capacity * 100) / acpi_last_full));
 			else
 				strncpy(last_battery_str, "AC", 64);
 		}
@@ -1348,7 +1351,22 @@ void get_battery_stuff(char *buf, unsigned int n, const char *bat)
 		}
 	}
 
-	snprintf(buf, n, "%s", last_battery_str);
+set_return_value:
+	switch (item) {
+        case BATTERY_STATUS:
+		{
+            		snprintf(buf, n, "%s", last_battery_str);
+            		break;
+          	}
+        case BATTERY_TIME:
+          	{
+            		snprintf(buf, n, "%s", last_battery_time_str);
+            		break;
+          	}
+	default:
+            		break;
+        }              
+	return;
 }
 
 /* On Apple powerbook and ibook:
@@ -1759,3 +1777,29 @@ Peter Tarjan (ptarjan@citromail.hu)
     return;
     
 }    
+
+void update_entropy (void)
+{
+    static int rep;
+    const char *entropy_avail = "/proc/sys/kernel/random/entropy_avail";
+    const char *entropy_poolsize = "/proc/sys/kernel/random/poolsize";
+    FILE *fp1, *fp2;
+
+    info.entropy.entropy_avail=0;
+    info.entropy.poolsize=0;
+
+    if ((fp1 = open_file (entropy_avail, &rep))==NULL)
+	return;
+
+    if ((fp2 = open_file (entropy_poolsize, &rep))==NULL)
+    {
+	fclose (fp1);
+	return;
+    }
+
+    fscanf (fp1, "%u", &info.entropy.entropy_avail);
+    fscanf (fp2, "%u", &info.entropy.poolsize);
+
+    fclose (fp1);
+    fclose (fp2);
+}
